import 'dart:convert';

import 'package:fis_jsonrpc/services/followUp.m.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:vitalapp/architecture/defines.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/managers/interfaces/follow_up.dart';
import 'package:vitalapp/pages/check/follow_up/controller.dart';
import 'package:vitalapp/pages/check/traditional_chinese_medicine_consitution_record/state.dart';
import 'package:vitalapp/pages/traditional_chinese_medicine_constitution/widget/ConfigurableCard.dart';
import 'package:vitalapp/store/store.dart';

class TraditionalChineseMedicineConsitutionRecordController
    extends FControllerBase {
  TraditionalChineseMedicineConsitutionRecordController();
  final state = TraditionalChineseMedicineConsitutionRecordState();
  final _followUpManager = Get.find<IFollowUpManager>();
  late String patientCode;
  late String patientName;
  late FollowUpController _followUpController;
  late String key;
  @override
  void onReady() {
    super.onReady();
    _initData();
  }

  _initData() {
    update(["contract_records"]);
    patientCode = Store.user.currentSelectPatientInfo?.code ?? '';
    patientName = Store.user.currentSelectPatientInfo?.patientName ?? '';
    final p = Get.parameters;
    key = p["key"] ?? "";
    _getFollowUpRecordList();
    _followUpController = Get.put(FollowUpController());
  }

  Future<void> _getFollowUpRecordList() async {
    try {
      var keys = [key];
      var result =
          await _followUpManager.getFollowUpRecordList(keys, patientCode);
      if (result != null) {
        state.followUpDTOList = result;
      }
    } catch (e) {
      return;
    }
  }

  Future<void> updateFollowUp(key, code, data) async {
    List<String> followUpPhotos = [];
    if (_followUpController.state.followUpPhoto?.isNotEmpty ?? false) {
      followUpPhotos = [_followUpController.state.followUpPhoto!];
    }
    final result = await _followUpManager.updateFollowUp(
      UpdateFollowUpRequest(
        key: key,
        followUpData: data,
        followUpTime: _followUpController.state.followUpTime,
        nextFollowUpTime: _followUpController.state.nextFollowUpTime,
        followUpMode: _followUpController.state.followUpMode ??
            FollowUpModeEnum.Outpatient,
        code: code,
        followUpPhotos: followUpPhotos,
      ),
    );
    if (result ?? false) {
      PromptBox.toast('保存成功');
    } else {
      PromptBox.toast('保存失败');
    }
  }

  Future<List> getFollowUpTableData(FollowUpRecordDTO dto, int index) async {
    var currentFollowUp = dto.followUpRecordDatas?[index];
    var followUpTableData = await _followUpManager
        .getFollowUpRecordListByYearAsync(GetFollowUpRecordListByYearRequest(
      year: (currentFollowUp?.followUpTime?.year ?? DateTime.now().year)
          .toString(),
      keys: [currentFollowUp?.key ?? ''],
      patientCode: patientCode,
    ));
    List tableData = [];

    followUpTableData?.first.followUpRecordDatas?.forEach((element) {
      Map followUp = {
        "Follow_Time": element.followUpTime != null
            ? DateFormat("yyyy-MM-dd").format(element.followUpTime!.toLocal())
            : "",
        "Follow_Type": '${(element.followUpMode?.index ?? 0) + 1} ',
        "Next_Follow_Up_Time": element.nextFollowUpTime != null
            ? DateFormat("yyyy-MM-dd")
                .format(element.nextFollowUpTime!.toLocal())
            : "",
      };
      followUp.addAll(jsonDecode(element.followUpData ?? ''));
      tableData.add(followUp);
    });

    return tableData;
  }

  toCheckPage(FollowUpRecordDataDTO dataDTO) async {
    _followUpController.state.followUpTime = dataDTO.followUpTime;
    _followUpController.state.nextFollowUpTime = dataDTO.nextFollowUpTime;
    _followUpController.state.followUpMode = dataDTO.followUpMode;
    Get.to(
      ConfigurableCard(
        cardKey: dataDTO.key!,
        examData: dataDTO.followUpData,
        patientCode: patientCode,
        isEdit: true,
        callBack: (key, code, data) async {
          await updateFollowUp(key, dataDTO.code, data);
          await _getFollowUpRecordList();
          return true;
        },
      ),
      transition: Transition.rightToLeft,
    );
  }
}
