import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/components/appbar.dart';
import 'package:vitalapp/components/button.dart';
import 'package:vitalapp/components/dialog_input.dart';
import 'package:vitalapp/components/dynamic_drawer.dart';
import 'package:vitalapp/managers/interfaces/cachedRecord.dart';
import 'package:vitalapp/managers/interfaces/template.dart';
import 'package:vitalapp/pages/check/models/form.dart';
import 'package:vitalapp/pages/check/widgets/exam_configurable/exam_health_guidance_check_box.dart';
import 'package:vitalapp/pages/check/widgets/exam_configurable/exam_input.dart';
import 'dart:math' as math;

import 'package:vitalapp/pages/check/widgets/exam_configurable/exam_radio.dart';
import 'package:vitalapp/pages/check/widgets/exam_configurable/exam_radio_score.dart';
import 'package:vitalapp/pages/check/widgets/device_controller.dart';
import 'package:vitalapp/store/store.dart';

class ConfigurableCard extends StatefulWidget {
  final String cardKey;
  final Future<bool> Function(String, String, dynamic) callBack;
  final String? patientCode;
  final String? examData;
  final bool? isEdit;
  const ConfigurableCard({
    super.key,
    required this.cardKey,
    required this.callBack,
    this.patientCode,
    this.examData,
    this.isEdit = false,
  });

  @override
  State<ConfigurableCard> createState() => _ConfigurableFormState();
}

class _ConfigurableFormState extends State<ConfigurableCard> {
  /// 当前最新的模板的键值对
  Map<String, dynamic> templateRelation = {};

  bool isFirstEnter = true;

  /// 当前模板数据
  List<FormObject> currentTemplate = [];

  /// 当前title的下标
  int currentTitleIndex = 0;

  Map<String, dynamic> formValue = {};
  var scaffoldKey = GlobalKey<ScaffoldState>();

  final _templateManager = Get.find<ITemplateManager>();
  final _cachedRecordManager = Get.find<ICachedRecordManager>();
  final arrowHeight = math.tan(120 / 180) * 19;
  Map currentTable = {};

  List<String> storeTypeList = [
    'Qi_Score',
    'Yang_Score',
    'Yin_Score',
    'Tan_Score',
    'Shi_Score',
    'Xue_Score',
    'Qiyu_Score',
    'Te_Score',
    'Ping_Score'
  ];

  @override
  void initState() {
    Get.put(DeviceController());

    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
      if (mounted) {
        initTemplate();
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: scaffoldKey,
      appBar: widget.isEdit! ? VAppBar(titleText: "中医体质") : null,
      endDrawer: VDynamicDrawerWrapper(scaffoldKey: scaffoldKey),
      resizeToAvoidBottomInset: false,
      body: (!widget.isEdit!) && isFirstEnter
          ? Column(
              children: [
                const SizedBox(
                  height: 50,
                ),
                const Center(
                  child: SizedBox(
                    width: 900,
                    child: Text(
                      '《中医体质分类与判定标准》于2009年4月9日发布，由中华中医药学会正式发布，该标准旨在为体质辨识及中医体质相关疾病的防治、养生保健、健康管理提供依据，使体质分类科学化、规范化，是我国第一部指导和规范中医体质研究及应用的文件。适用于从事中医体质研究的中医临床医生、科研人员及相关管理人员，并可作为临床实践、判定规范及质量评定的重要参考依据。其中将中国人的体制分为九种，是体质分类的一种规范。具体有平和质、阳虚质、气虚质、痰湿质、湿热质、气郁质、血瘀质、特禀质。',
                      style: TextStyle(fontSize: 24),
                    ),
                  ),
                ),
                const SizedBox(
                  height: 50,
                ),
                VButton(
                  label: "开始检测",
                  onTap: () {
                    setState(() {
                      isFirstEnter = false;
                    });
                  },
                ),
              ],
            )
          : Column(
              children: [
                const SizedBox(
                  height: 20,
                ),
                Expanded(
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      const SizedBox(width: 50),
                      _buildContent(),
                      const SizedBox(width: 50),
                    ],
                  ),
                ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    if (currentTitleIndex == 0 && widget.patientCode != "")
                      VButton(
                        onTap: _buildCompute,
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: const [
                            Text("查看体质", style: TextStyle(fontSize: 20)),
                          ],
                        ),
                      ),
                    const SizedBox(
                      width: 10,
                    ),
                    if (currentTitleIndex == 1)
                      VButton(
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: const [
                            Text("返回", style: TextStyle(fontSize: 20)),
                          ],
                        ),
                        onTap: () {
                          setState(() {
                            // if (!widget.isEdit!) formValue.clear();
                            currentTitleIndex = 0;
                          });
                        },
                      ),
                    const SizedBox(
                      width: 10,
                    ),
                    if (currentTitleIndex == 1)
                      VButton(
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: const [
                            Text("提交", style: TextStyle(fontSize: 20)),
                          ],
                        ),
                        onTap: () async {
                          final result = await widget.callBack(
                            widget.cardKey,
                            templateRelation[widget.cardKey]!,
                            jsonEncode(formValue),
                          );
                          if (result) {
                            setState(() {
                              if (!widget.isEdit!) formValue.clear();
                              currentTitleIndex = 0;
                            });
                          }
                        },
                      ),
                  ],
                ),
              ],
            ),
    );
  }

  @override
  void dispose() {
    super.dispose();
  }

  Future<void> initTemplate() async {
    Store.app.busy = true;
    await fetchTemplateIndex();
    await fetchTemplate(widget.cardKey);
    await fetchTemplateData();
    Store.app.busy = false;
  }

  /// 读取体检的缓存
  Future<String?> readCachedCheck() async {
    if (widget.patientCode == null) {
      return null;
    }
    String? value = await _cachedRecordManager.readCachedRecord(
      widget.cardKey,
      widget.patientCode!,
      'FollowUp',
    );
    return value;
  }

  Future<void> fetchTemplateIndex() async {
    try {
      /// 获取模板的键值对
      String? templates;
      templates = await _templateManager.readTemplate('templateRelation');
      templateRelation = jsonDecode(templates!);

      setState(() {});
    } catch (error) {
      print('发生错误: $error');
    }
  }

  Future<void> fetchTemplateData() async {
    // / 这逻辑需要优化
    if (widget.examData?.isNotEmpty ?? false) {
      formValue = jsonDecode(widget.examData!);
      return;
    }
    // String? value = await readCachedCheck();
    // if (value?.isNotEmpty ?? false) {
    //   formValue = jsonDecode(value!);
    // }

    formValue.forEach(
      (key, value) {
        if (value is List<String>) {
          formValue[key] = List<String>.from(formValue[key]);
        } else if (value is List<Map>) {
          formValue[key] = List<Map>.from(formValue[key]);
        }
      },
    );
    setState(() {});
  }

  Future<void> fetchTemplate(String key) async {
    try {
      if (templateRelation[key] == null) {
        currentTemplate = [];
        setState(() {});
        return;
      }
      var template =
          await _templateManager.readTemplate(templateRelation[key]!);
      String templateContent =
          TemplateDTO.fromJson(jsonDecode(template!)).templateContent!;
      List<Map<String, dynamic>> list =
          jsonDecode(templateContent).cast<Map<String, dynamic>>();
      for (var i in list) {
        if (i['children'] != null) {
          List<FormObject> currentChildren = [];
          for (var j in i['children']) {
            currentChildren.add(FormObject.fromJson(j));
          }
          i['children'] = currentChildren;
        }
        var item = FormObject.fromJson(i);
        currentTemplate.add(item);
      }
      if (widget.cardKey == 'LNRZYYJKGLFWJL') {
        for (var element in storeTypeList) {
          formValue[element] =
              calculatePhysicalFitnessScore(element, element == 'Ping_Score');
        }
      }

      setState(() {});
    } catch (error) {
      print('发生错误: $error');
    }
  }

  Widget buildSingleItem(Widget item, int span) {
    return Column(
      children: [
        FractionallySizedBox(
          widthFactor: span == 24 ? 1 : 0.5,
          child: item,
        ),
      ],
    );
  }

  Widget buildWidget(FormObject? currentFormObject) {
    Map<String, Widget Function(FormObject)> widgetMap = {
      'input': _buildInput,
      'radio': _buildRadio,
      'radioScore': _buildRadioScore,
      'healthGuidanceCheckBox': _buildHealthGuidanceCheckBox,
    };
    Widget Function(FormObject) builder =
        widgetMap[currentFormObject?.type] ?? _buildInput;
    return builder(currentFormObject!);
  }

  Widget flowCardList() {
    int itemCount = 0;
    bool currentTemplateOptionsIsNotEmpty = false;
    if (currentTemplate.isNotEmpty) {
      itemCount = currentTemplate[currentTitleIndex].children?.length ?? 0;
      currentTemplateOptionsIsNotEmpty =
          currentTemplate[currentTitleIndex].options?.isNotEmpty ?? false;
    }

    List<Widget> items = List.generate(itemCount, (index) {
      FormObject? currentFormObject =
          currentTemplate[currentTitleIndex].children?[index];
      int span = currentFormObject?.span ?? 12;
      //父结构的options不等于空或null
      if (true) {
        //子结构的options若是无值则取父类的options值
        if (currentTemplateOptionsIsNotEmpty) {
          currentFormObject!.options =
              currentTemplate[currentTitleIndex].options;
        }
      }
      return buildSingleItem(buildWidget(currentFormObject), span);
    });

    return Scrollbar(
      thumbVisibility: true,
      child: SingleChildScrollView(
        child: Container(
          alignment: Alignment.topCenter,
          padding: const EdgeInsets.all(15),
          child: Wrap(
            runSpacing: 20, // 纵向元素间距
            alignment: WrapAlignment.start,
            children: items,
          ),
        ),
      ),
    );
  }

  /// title标签
  _buildCompute() {
    // currentTemplate.asMap().entries.map((e) {
    if (widget.cardKey == 'LNRZYYJKGLFWJL' &&
        currentTitleIndex == 0 &&
        formValue.length >= 33) {
      setState(() {
        currentTitleIndex = 1;
      });
    } else {
      PromptBox.toast('题目未答完整，无法计算体质');
    }
    // });
  }

  /// 主页面
  Widget _buildContent() {
    return Expanded(
      flex: 2,
      child: flowCardList(),
    );
  }

  Widget _buildInput(FormObject currentFormObject) {
    String currentInputValue = formValue[currentFormObject.key!] ?? '';
    Future<void> commonInput() async {
      String? result = await VDialogInput(
        title: currentFormObject.label,
        initialValue: formValue[currentFormObject.key],
      ).show();
      if (result?.isNotEmpty ?? false) {
        formValue[currentFormObject.key!] = result;
        currentInputValue = formValue[currentFormObject.key!];
        setState(() {});
      }
    }

    return ExamInput(
      currentInputValue: currentInputValue,
      commonInput: commonInput,
      currentFormObject: currentFormObject,
    );
  }

  ///中医药健康指导多选框组件
  Widget _buildHealthGuidanceCheckBox(FormObject currentFormObject) {
    List<Option> options = currentFormObject.options ?? [];
    List<dynamic> currentSelectedCheckBox =
        formValue[currentFormObject.key!] ?? [];
    var currentScoreKey = currentFormObject.groupKeys != null
        ? currentFormObject.groupKeys![0]
        : '';
    int score = formValue[currentScoreKey] ?? 0;
    String judgmentResult = getJudgmentResult(currentScoreKey);
    void selectCheckBoxChange(Option e) {
      if (currentSelectedCheckBox.contains(e.value)) {
        currentSelectedCheckBox.remove(e.value);
      } else {
        currentSelectedCheckBox.add(e.value ?? '');
      }
      formValue[currentFormObject.key!] = currentSelectedCheckBox;

      setState(() {});
    }

    if (["是", "基本是", "倾向是"].contains(judgmentResult)) {
      return ExamHealthGuidanceCheckBox(
        options: options,
        currentSelectedCheckBox: currentSelectedCheckBox,
        currentFormObject: currentFormObject,
        selectCheckBoxChange: selectCheckBoxChange,
        score: score,
        judgmentResult: judgmentResult,
      );
    } else {
      return Container();
    }
  }

  ///识别体质
  String getJudgmentResult(String storeKey) {
    var score = formValue[storeKey] ?? 0;
    if (storeKey == 'Ping_Score') {
      if (score >= 17) {
        bool is8 = true;
        bool is10 = true;
        for (var e in storeTypeList) {
          if (formValue[e] != null && e != 'Ping_Score') {
            var otherScore = formValue[e];
            if (otherScore > 8) {
              if (is8) {
                is8 = false;
              }
              if (otherScore > 10) {
                is10 = false;
              }
            }
            if (!is10) {
              break;
            }
          }
        }
        if (is8) {
          return '是';
        } else if (is10) {
          return '基本是';
        }
      }
      return '否';
    } else {
      if (score >= 11) {
        return '是';
      } else if (9 == score && score == 10) {
        return '倾向是';
      } else {
        return '否';
      }
    }
  }

  /// 单选框组件
  Widget _buildRadio(FormObject currentFormObject) {
    List<Option> options = currentFormObject.options ?? [];
    String currentSelected = formValue[currentFormObject.key!] ?? "";

    void selectRaidoChange(Option e) {
      currentSelected = e.value ?? '';
      formValue[currentFormObject.key!] = currentSelected;
      if (widget.cardKey == 'LNRZYYJKGLFWJL') {
        var pingScore = [
          'qusition1',
          'qusition2',
          'qusition4',
          'qusition5',
          'qusition13'
        ];
        var isPingScore = pingScore.contains(currentFormObject.key);
        formValue[currentFormObject.groupKeys?[0]] =
            calculatePhysicalFitnessScore(
                currentFormObject.groupKeys?[0], false);
        if (isPingScore) {
          formValue['Ping_Score'] =
              calculatePhysicalFitnessScore('Ping_Score', isPingScore);
        }
      }
      setState(() {});
    }

    return ExamRadio(
      options: options,
      currentFormObject: currentFormObject,
      selectRaidoChange: selectRaidoChange,
      currentSelected: currentSelected,
    );
  }

  ///计算得分
  int calculatePhysicalFitnessScore(String currentScoreKey, bool isPingScore) {
    int score = 0;
    var currentTemplateStoreList = currentTemplate[0].children?.where(
        (element) =>
            element.groupKeys != null &&
            element.groupKeys!.contains(currentScoreKey));
    if (!isPingScore) {
      for (var element in currentTemplateStoreList!) {
        if (formValue[element.key!] != null) {
          score += int.parse(formValue[element.key!]!);
        }
      }
    } else {
      int index = 0;
      for (var element in currentTemplateStoreList!) {
        if (index == 0) {
          if (formValue[element.key!] != null) {
            score += int.parse(formValue[element.key!]!);
          }
        } else {
          score += formValue[element.key!] != null
              ? (6 - int.parse(formValue[element.key!]!))
              : 0;
        }
        index++;
      }
    }

    return score;
  }

  Widget _buildRadioScore(FormObject currentFormObject) {
    print(currentFormObject.toJson());
    List<Option> options = currentFormObject.options ?? [];
    String currentSelected =
        formValue[currentFormObject.childrenKey!.first] ?? "";
    String currentScore = formValue[currentFormObject.childrenKey!.last] ?? "";

    void selectRaidoChange(Option e) {
      currentSelected = e.value ?? '';
      formValue[currentFormObject.childrenKey!.first] = currentSelected;
      setState(() {});
    }

    void changeScore(String? score) {
      currentScore = score ?? '';
      formValue[currentFormObject.childrenKey!.last] = currentScore;
      setState(() {});
    }

    return ExamRadioScore(
      options: options,
      currentFormObject: currentFormObject,
      selectRaidoChange: selectRaidoChange,
      currentSelected: currentSelected,
      changeScore: changeScore,
      currentScore: currentScore,
    );
  }
}
